// code found on forum - re-written
// written by amateur coder - use at own risk
// author accepts no resposibility for anything, anywhere, ever

// timer for PIC16F886 with 4MHz crystal
// HS oscillator, watchdog off

// 1)I need to make my START_STOP button when pressed a second time to stop the counter and resets.
// 30min_button is ON
// Start_Stop_button is ON
// Counter starts to count down
// If Start_Stop_button is pressed again counter resets.
//
// 2) when insignal Oven_OFF is received Counter stops and resets.

//###################################################################################
 //Needs to Fix:
 //1)START_STOP_Button when pressed stops the timer
 //2)Time on display in this format HH:MM -- Fixed
 //3)Blink on LCD display needs to work when timer is ON
 //4)led_RED is ON when Start_Stop button is pressed --Fixed
 //5)Led_Gr/Yel or Blue need to Blink every sec when ON (Start_Stop) is pressed
 //6)When Enable signal from IR is active then all Relays close and timer resets
 //7) Date: 2019-02-26
//###################################################################################

// LCD module connections
sbit LCD_RS at RB2_bit;
sbit LCD_EN at RB3_bit;
sbit LCD_D4 at RB4_bit;
sbit LCD_D5 at RB5_bit;
sbit LCD_D6 at RB6_bit;
sbit LCD_D7 at RB7_bit;
sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;

sbit Relay_1 at RA0_bit;
sbit Relay_2 at RA1_bit;
sbit Relay_3 at RA2_bit;

//Outputs:LEDs
sbit led_RED at RC5_bit;                                                        // led Red for Start/Stop of timers
sbit led_GREEN at RA4_bit;                                                      // led Green for 30 mins
sbit led_YELLOW at RA5_bit;                                                     // led Yellow for 3 hrs
sbit led_BLUE at RB0_bit;                                                       // led Blue for 8 hrs

//Outputs: Buzzer
sbit Buzzer at RB1_bit; //Buzzer

//Inputs: Tact Switches and external signal
sbit Start_Stop_Button at RC0_bit;                                              // Starts or Stops Timer Select
sbit Min30_Button at RC1_bit;                                                   // Set 30 min timer
sbit Hour2_Button at RC2_bit;                                                   // Set 3 hours timer
sbit Hour5_Button at RA3_bit;                                                   // Set 8 hours timer
sbit Oven_OFF at RC7_bit;                                                       // Close oven and all 3Relays

//sbit Off_3Relays at RA4_bit; // Shutdown all 3 relays
// Messages
char Message1[] = "30min-5hr:Timer OFF";
char Message2[] = "Timer OFF";
char Message3[] = "Timer ON: HH:MM";
char Message4[] = "Set Time: ";
char Message5[] = "Time Left: ";

// global count-down store
volatile signed char fraction;                                                  // 5ths of a second
volatile signed char seconds_units;                                             // volatile to warn compiler that value may change in interrupt
volatile signed char seconds_tens;
volatile signed char minutes_units;
volatile signed char minutes_tens;
volatile signed char hours;

// other global variables
unsigned char digit[5] = "0:00";
volatile unsigned char update_flag;                                             // set to show display update needed
volatile unsigned char count_flag;                                              // set when counting down is wanted
volatile unsigned char zero_flag;                                               // set when zero count is reached
unsigned char init_flag;                                                        // set when in initialised state

//Timer1 Initialise Prescaler 1:2; TMR1 Preload = 15536; Actual Interrupt Time : 200 ms
void InitTimer1(){
    T1CON         = 0x21;
    TMR1IF_bit         = 0;
    TMR1H         = 0x3C;
    TMR1L         = 0xB0;
    TMR1IE_bit         = 1;
    INTCON         = 0xC0;
}

// interrupt every 200 milliseconds
void interrupt(){
    if (TMR1IF_bit){
        TMR1IF_bit = 0;
        TMR1H         = 0x3C;
        TMR1L         = 0xB0;

        if(count_flag){                                                         // only count down while flag is set
            fraction--;                                                         // count down
            if(fraction | seconds_units | seconds_tens | minutes_units | minutes_tens | hours){  // check for non-zero
                if(fraction < 0){                                               // must have wrapped around
                    fraction = 4;
                    Relay_3 = ~Relay_3;                                         // toggle RA2 every second
                    update_flag = 1;                                            // time to display new number
                    seconds_units--;
                    if(seconds_units < 0){                                      // must have wrapped around
                        seconds_units = 9;

                        seconds_tens--;
                        if(seconds_tens < 0){                                   // must have wrapped around
                            seconds_tens = 5;

                            minutes_units--;
                            if(minutes_units < 0){                              // must have wrapped around
                                minutes_units = 9;

                                minutes_tens--;
                                if(minutes_tens < 0){                           // must have wrapped around
                                    minutes_tens = 5;
                                }
                            }
                        }
                    }
                }
            }
            else{                                                               // counter is zero
                count_flag = 0;                                                 // prevent going below zero
                zero_flag = 1;                                                  // so indicate zero reached
            }
        }
    }
}

// button de-douce delay
void debounce(){
    Delay_ms(250);
}

// all LEDs off
void leds_off(){
    led_BLUE = 0;                                                               // RB0
    led_RED = 0;                                                                // RC5
    led_GREEN = 0;                                                              // RA4
    led_YELLOW = 0;                                                             // RA5
}

// Buzzer sound - piezo Buzzer on RB1
void Play_Sound(){
  unsigned int buzz_count;

    for(buzz_count = 0; buzz_count < 1000; buzz_count++){                       // 1000 times 1 ms gives 1 second buzz
        Buzzer = 1;
        Delay_us(500);                                                          // adjust on/off times for preferred volume and frequency
        Buzzer = 0;
        Delay_us(500);                                                          // off for remainder of 1ms to give 1kHz
    }
}

// turn relays on
void relays_on(){
    Relay_1 = 1;
    Relay_2 = 1;
    Relay_3 = 1;
    Lcd_Out(1,1,Message3);                                                      // "Timer ON: HH:MM"
}

// set time to zero and stop counting
void time_zero(){

    count_flag = 0;                                                             // stop the countdown
    fraction = 0;                                                               // set all time variables to zero
    seconds_units = 0;
    seconds_tens = 0;
    minutes_units = 0;
    minutes_tens = 0;
    hours = 0;
    init_flag = 0;                                                              // show that we are not in initialised (time set) state
    zero_flag = 0;
    Relay_1 = 0;                                                                // RA0
    Relay_2 = 0;                                                                // RA1
    Relay_3 = 0;                                                                // RA2
    leds_off();                                                                 // all LEDs off
    Lcd_Out(1,1,Message2);                                                      // "Timer OFF"
}

void main() {

    count_flag = 0;                                                             // no count down to start with
    ANSEL = 0x00;                                                               // all digital
    ANSELH = 0x00;                                                              // all digital
    CM1CON0 = 7;                                                                // Disable Comparators
    TRISA = 0b11001000;                                                         // TRISA
    TRISB = 0b00000000;                                                         // TRISB
    TRISC = 0b11001111;                                                         // TRISC
    Relay_1 = 0;                                                                // RA0
    Relay_2 = 0;                                                                // RA1
    Relay_3 = 0;                                                                // RA2
    Lcd_Init();                                                                 // Initialize LCD

    Play_Sound();
    relays_on();
    leds_off();                                                                 // all LEDs off
    led_RED = 1;
    Lcd_Cmd(_LCD_CLEAR);                                                        // Clear LCD
    Lcd_Out(2,1,Message5);                                                      // "Time Left: "
    OPTION_REG = 0x80 ;                                                         // bit7 Enable PORTB Pull UP

    time_zero();                                                                // set time to zero and stop counting

    InitTimer1();                                                               // start 100ms timer Interrupt running
    update_flag = 1;                                                            // show initial value
    
    while(1){                                                                   // main loop

        while(!update_flag){                                                    // wait for interrupt routine to indicate display update needed
            if(!Start_Stop_Button){                                             // and while waiting - check buttons
                if(count_flag){                                                 // are we counting?
                    count_flag = 0;                                             // then stop counting
                    init_flag = 0;                                              // next buton press causes reset
                }
                else{                                                           // not counting....
                    if(init_flag){                                              // are we in initialised state?
                        count_flag = 1;                                         // then start counting
                        relays_on();                                            // turn on all relays
                    }
                    else{
                        time_zero();                                            // reset if not counting and not initialised with time set
                    }
                }
                debounce();                                                     // prevent multiple triggers
                while(!Start_Stop_Button);                                      // wait for button release
                update_flag = 1;                                                // show new value
            }
            if(!Min30_Button){                                                  // if RC1 goes low
                leds_off();                                                     // all lEDs off
                led_GREEN = 1;
                fraction = 0;
                seconds_units = 0;
                seconds_tens = 1;
                minutes_units = 0;
                minutes_tens = 0;
                hours = 0;
                debounce();                                                     // prevent multiple triggers
                while(!Min30_Button);                                           // wait for button release
                update_flag = 1;                                                // show new value
                init_flag = 1;                                                  // now in initialised state (time is set)
            }
            if(!Hour2_Button){                                                  // if RC2 goes low
                leds_off();                                                     // all lEDs off
                led_YELLOW = 1;                                                 // RA5
                fraction = 0;
                seconds_units = 0;
                seconds_tens = 0;
                minutes_units = 0;
                minutes_tens = 0;
                hours = 2;
                debounce();                                                     // prevent multiple triggers
                while(!Hour2_Button);                                           // wait for button release
                update_flag = 1;                                                // show new value
                init_flag = 1;                                                  // now in initialised state (time is set)
            }
            if(!Hour5_Button){                                                  // if RC2 goes low
                leds_off();                                                     // all lEDs off
                led_BLUE = 1;                                                   // RB0
                fraction = 0;
                seconds_units = 0;
                seconds_tens = 0;
                minutes_units = 0;
                minutes_tens = 0;
                hours = 2;
                debounce();                                                     // prevent multiple triggers
                while(!Hour5_Button);                                           // wait for button release
                update_flag = 1;                                                // show new value
                init_flag = 1;                                                  // now in initialised state (time is set)
            }
            if(!Oven_OFF){                                                      // if RC7 goes low
                time_zero();                                                    // set time to zero and stop counting
                debounce();                                                     // prevent multiple triggers
            }
            if(zero_flag){                                                      // countdown just reached zero
                time_zero();                                                    // set time to zero and stop counting
                Play_Sound();                                                   // activate the buzzer
            }
        }
        
        Lcd_Chr(2, 15, seconds_units + 48);                                     // print seconds
        Lcd_Chr(2, 14, seconds_tens + 48);
        Lcd_Chr(2, 13, ':');                                                    // print colon
        Lcd_Chr(2, 12, minutes_units + 48);                                     // print minutess
        Lcd_Chr(2, 11, minutes_tens + 48);                                      // print minutess
        update_flag = 0;                                                        // prevent multiple printouts
    }
}