#line 1 "C:/PIC/MikroC/PIC16/pic16f886/timer/timer.c"
#line 28 "C:/PIC/MikroC/PIC16/pic16f886/timer/timer.c"
sbit LCD_RS at RB2_bit;
sbit LCD_EN at RB3_bit;
sbit LCD_D4 at RB4_bit;
sbit LCD_D5 at RB5_bit;
sbit LCD_D6 at RB6_bit;
sbit LCD_D7 at RB7_bit;
sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;

sbit Relay_1 at RA0_bit;
sbit Relay_2 at RA1_bit;
sbit Relay_3 at RA2_bit;


sbit led_RED at RC5_bit;
sbit led_GREEN at RA4_bit;
sbit led_YELLOW at RA5_bit;
sbit led_BLUE at RB0_bit;


sbit Buzzer at RB1_bit;


sbit Start_Stop_Button at RC0_bit;
sbit Min30_Button at RC1_bit;
sbit Hour2_Button at RC2_bit;
sbit Hour5_Button at RA3_bit;
sbit Oven_OFF at RC7_bit;



char Message1[] = "30min-5hr:Timer OFF";
char Message2[] = "Timer OFF";
char Message3[] = "Timer ON: HH:MM";
char Message4[] = "Set Time: ";
char Message5[] = "Time Left: ";


volatile signed char fraction;
volatile signed char seconds_units;
volatile signed char seconds_tens;
volatile signed char minutes_units;
volatile signed char minutes_tens;
volatile signed char hours;


unsigned char digit[5] = "0:00";
volatile unsigned char update_flag;
volatile unsigned char count_flag;
volatile unsigned char zero_flag;
unsigned char init_flag;


void InitTimer1(){
 T1CON = 0x21;
 TMR1IF_bit = 0;
 TMR1H = 0x3C;
 TMR1L = 0xB0;
 TMR1IE_bit = 1;
 INTCON = 0xC0;
}


void interrupt(){
 if (TMR1IF_bit){
 TMR1IF_bit = 0;
 TMR1H = 0x3C;
 TMR1L = 0xB0;

 if(count_flag){
 fraction--;
 if(fraction | seconds_units | seconds_tens | minutes_units | minutes_tens | hours){
 if(fraction < 0){
 fraction = 4;
 Relay_3 = ~Relay_3;
 update_flag = 1;
 seconds_units--;
 if(seconds_units < 0){
 seconds_units = 9;

 seconds_tens--;
 if(seconds_tens < 0){
 seconds_tens = 5;

 minutes_units--;
 if(minutes_units < 0){
 minutes_units = 9;

 minutes_tens--;
 if(minutes_tens < 0){
 minutes_tens = 5;
 }
 }
 }
 }
 }
 }
 else{
 count_flag = 0;
 zero_flag = 1;
 }
 }
 }
}


void debounce(){
 Delay_ms(250);
}


void leds_off(){
 led_BLUE = 0;
 led_RED = 0;
 led_GREEN = 0;
 led_YELLOW = 0;
}


void Play_Sound(){
 unsigned int buzz_count;

 for(buzz_count = 0; buzz_count < 1000; buzz_count++){
 Buzzer = 1;
 Delay_us(500);
 Buzzer = 0;
 Delay_us(500);
 }
}


void relays_on(){
 Relay_1 = 1;
 Relay_2 = 1;
 Relay_3 = 1;
 Lcd_Out(1,1,Message3);
}


void time_zero(){

 count_flag = 0;
 fraction = 0;
 seconds_units = 0;
 seconds_tens = 0;
 minutes_units = 0;
 minutes_tens = 0;
 hours = 0;
 init_flag = 0;
 zero_flag = 0;
 Relay_1 = 0;
 Relay_2 = 0;
 Relay_3 = 0;
 leds_off();
 Lcd_Out(1,1,Message2);
}

void main() {

 count_flag = 0;
 ANSEL = 0x00;
 ANSELH = 0x00;
 CM1CON0 = 7;
 TRISA = 0b11001000;
 TRISB = 0b00000000;
 TRISC = 0b11001111;
 Relay_1 = 0;
 Relay_2 = 0;
 Relay_3 = 0;
 Lcd_Init();

 Play_Sound();
 relays_on();
 leds_off();
 led_RED = 1;
 Lcd_Cmd(_LCD_CLEAR);
 Lcd_Out(2,1,Message5);
 OPTION_REG = 0x80 ;

 time_zero();

 InitTimer1();
 update_flag = 1;

 while(1){

 while(!update_flag){
 if(!Start_Stop_Button){
 if(count_flag){
 count_flag = 0;
 init_flag = 0;
 }
 else{
 if(init_flag){
 count_flag = 1;
 relays_on();
 }
 else{
 time_zero();
 }
 }
 debounce();
 while(!Start_Stop_Button);
 update_flag = 1;
 }
 if(!Min30_Button){
 leds_off();
 led_GREEN = 1;
 fraction = 0;
 seconds_units = 0;
 seconds_tens = 1;
 minutes_units = 0;
 minutes_tens = 0;
 hours = 0;
 debounce();
 while(!Min30_Button);
 update_flag = 1;
 init_flag = 1;
 }
 if(!Hour2_Button){
 leds_off();
 led_YELLOW = 1;
 fraction = 0;
 seconds_units = 0;
 seconds_tens = 0;
 minutes_units = 0;
 minutes_tens = 0;
 hours = 2;
 debounce();
 while(!Hour2_Button);
 update_flag = 1;
 init_flag = 1;
 }
 if(!Hour5_Button){
 leds_off();
 led_BLUE = 1;
 fraction = 0;
 seconds_units = 0;
 seconds_tens = 0;
 minutes_units = 0;
 minutes_tens = 0;
 hours = 2;
 debounce();
 while(!Hour5_Button);
 update_flag = 1;
 init_flag = 1;
 }
 if(!Oven_OFF){
 time_zero();
 debounce();
 }
 if(zero_flag){
 time_zero();
 Play_Sound();
 }
 }

 Lcd_Chr(2, 15, seconds_units + 48);
 Lcd_Chr(2, 14, seconds_tens + 48);
 Lcd_Chr(2, 13, ':');
 Lcd_Chr(2, 12, minutes_units + 48);
 Lcd_Chr(2, 11, minutes_tens + 48);
 update_flag = 0;
 }
}
